-- We will store the interface manager object here so that we don't have to make multiple GetInterfaceManager calls.
local interfaceMngr = nil;

-- The amazonForm table allows us to store all objects related to the specific form inside the table so that we can easily prevent naming conflicts if we need to add more than one form and track elements from both.
local amazonForm = {};
amazonForm.Form = nil;
amazonForm.RibbonPage = nil;
amazonForm.Browser = nil;
amazonForm.ImportButton = nil;

-- Retrieve our settings and store them in local variables to prevent redundant fetching of settings.  The creation of a table is not necessary, but like the form table above, it makes prevention of naming conflicts easier.
local settings = {}
settings.AutoSearch = GetSetting("AutoSearch");
settings.PriceField = GetSetting("PriceField");
settings.ImportKindlePrices = GetSetting("ImportKindlePrices");
settings.ImportUsedPrices = GetSetting("ImportUsedPrices");

require "Atlas.AtlasHelpers";

local browserType = nil;
if AddonInfo.Browsers and AddonInfo.Browsers.WebView2 then
    browserType = "WebView2";
else
    browserType = "Chromium";
end

function Init()
	if GetFieldValue("Transaction", "RequestType") == "Loan" then
		interfaceMngr = GetInterfaceManager();
		
		-- Create a form
		amazonForm.Form = interfaceMngr:CreateForm("Amazon", "Script");
		
		-- Add a browser
		amazonForm.Browser = amazonForm.Form:CreateBrowser("Amazon Books", "Amazon Browser", "Amazon Books", browserType);
		
		-- Hide the text label
		amazonForm.Browser.TextVisible = false;
		
		-- Since we didn't create a ribbon explicitly before creating our browser, it will have created one using the name we passed the CreateBrowser method.  We can retrieve that one and add our buttons to it.
		amazonForm.RibbonPage = amazonForm.Form:GetRibbonPage("Amazon Books");
		
		-- Create the search and import buttons.  We are storing the import button so we can enable it and disable it as needed.
		amazonForm.RibbonPage:CreateButton("Search", GetClientImage("Search32"), "Search", "Amazon");
		amazonForm.ImportButton = amazonForm.RibbonPage:CreateButton("Import Price", GetClientImage("ImportData32"), "ImportPrice", "Amazon");
		amazonForm.ImportButton.BarButton.Enabled = false;
		
		-- After we add all of our buttons and form elements, we can show the form.
		amazonForm.Form:Show();
		
		if settings.AutoSearch then
			Search();
		end
	end
end

function Search()
	amazonForm.Browser:RegisterPageHandler("custom", "CheckItemPageLoaded", "ItemPageLoaded", false);
	amazonForm.Browser:Navigate("https://www.amazon.com/s?k=".. AtlasHelpers.UrlEncode(GetFieldValue("Transaction", "LoanTitle")) .. "&i=stripbooks");
end

function CheckItemPageLoaded()
	local price = GetPrice();

	if IsNilOrBlank(price) then
		return false;
	end

	return true;
end

function CheckItemPageLeft()
	return CheckItemPageLoaded() == false;
end

-- We use the ItemPageLeft and ItemPageLoaded handlers to provide enable/disable functionality for the import button since we only want it enabled while on the item page.
function ItemPageLeft()
	amazonForm.ImportButton.BarButton.Enabled = false;
	amazonForm.Browser:RegisterPageHandler("custom", "CheckItemPageLoaded", "ItemPageLoaded", false);
end

function ItemPageLoaded()
	amazonForm.ImportButton.BarButton.Enabled = true;
	amazonForm.Browser:RegisterPageHandler("custom", "CheckItemPageLeft", "ItemPageLeft", true);
end

function ImportPrice()
	local price = GetPrice();

	if not price then
		return;
	end
	
	-- string.sub and match cut off currency symbol and any additional prices.
	SetFieldValue("Transaction", settings.PriceField, string.sub(price, 2):match("[%d%.]+"));
	ExecuteCommand("SwitchTab", {"Detail"});
end

function GetPrice()
	local price = nil;
	
	if settings.ImportUsedPrices then
		price = amazonForm.Browser:EvaluateScript("document.getElementById('usedPrice').innerText").Result;
	end

	if IsNilOrBlank(price) then
		price = amazonForm.Browser:EvaluateScript("document.getElementById('price').innerText").Result;
	end

	if IsNilOrBlank(price) then
		price = amazonForm.Browser:EvaluateScript("document.getElementById('newBuyBoxPrice').innerText").Result;
	end

	if IsNilOrBlank(price) and settings.ImportKindlePrices then
		price = amazonForm.Browser:EvaluateScript("document.getElementById('kindle-price').innerText").Result;
	end

	-- These are last resorts if none of the above work, because Amazon doesn't always have an ID directly associated with the element containing the price.
	if IsNilOrBlank(price) then
		price = amazonForm.Browser:EvaluateScript("document.getElementsByClassName('a-size-base a-color-price offer-price a-text-normal')[0].innerText").Result;
	end

	if IsNilOrBlank(price) then
		price = amazonForm.Browser:EvaluateScript("document.getElementsByClassName('a-price a-text-price header-price a-size-base a-text-normal')[0].innerText").Result;
	end

	return price;
end

function IsNilOrBlank(str)
	if str and str ~= "" then
		return false;
	end

	return true;
end